// Constraints.cpp: implementation of the Constraints class.
//
//////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
// Disable the warning n. 4786
#pragma warning( disable : 4786 )
#endif

#include "Constraints.h"

namespace ucf
{

//////////////////////////////////////////////////////////////////////
// class Constraints
//////////////////////////////////////////////////////////////////////

Constraints::Constraints()
	:fNumRows(0)
	,fVariables(NULL)

	,fProblemInstance(NULL)
	,fFirstFreeActor(0)
	,fActors(8, static_cast<tActorModification*>(NULL)) // preallocation
{

}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

Constraints::~Constraints()
{

}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::Initialize(
		const Variables* variables,
		const UCProblemInstance* problemInstance)
{
	fVariables = variables;
	fProblemInstance = problemInstance;
	assert (fNumRows == 0);
	assert (fFirstFreeActor == 0);
}
//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::SetNumRows(tIndex numRows)
{
	assert (numRows>=0);
	fNumRows = numRows;
	if (fGlobalNames.capacity()<numRows)
		fGlobalNames.reserve(numRows);
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

Constraints::tIndex Constraints::GetNonZero() const
{
	tIndex nonZero=0;
	for (tIndex i=0; i<this->GetNumRows(); ++i)
	{
		tSparseVector coefficients;
		eConstrType constrType;
		tNumber rhs;
		tNumber range;
		this->GetConstraint(i, coefficients, constrType, rhs, range);
		nonZero += coefficients.GetBaseDim()+1; // rhs => +1
	}

	return nonZero;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::SetConstraintGlobalIndex(
			tIndex wichConstr,
			int globalIndex)
{
	assert (wichConstr>=0);
	assert (wichConstr<fNumRows);
	fGlobalNames[wichConstr] = globalIndex;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::ExtractDuals(
		const tDenseVector& allDuals,
		tSparseVector& constrDuals) const
{
	int nonZero=0;
	int i;
	for (i=0; i<fNumRows; ++i)
	{
		tSparseVector::tElementType dual = allDuals[fGlobalNames[i]];
		if (dual!=0)
			++nonZero;
	}

	constrDuals.Resize(nonZero, fNumRows);
	int nzIndex = 0; // Non Zero Index
	for (i=0; i<fNumRows; ++i)
	{
		tSparseVector::tElementType dual = allDuals[fGlobalNames[i]];
		if (dual!=0)
		{
			constrDuals.NonZero(nzIndex) = dual;
			constrDuals.Base(nzIndex) = i;
		}
	}
}
		
//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

Constraints::tIndex Constraints::RegisterActor()
{
	if (fFirstFreeActor>=int(fActors.size()))
	{
		fActors.resize(fActors.size()*2, NULL);
	};

	fActors[fFirstFreeActor] = new tActorModification;

	return fFirstFreeActor++;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::GetModification(
		tIndex actor, tModifications*& modifications) const
{
	assert (actor>=0 && actor<tIndex(fActors.size()));
	fActors[actor]->GetModification(modifications);
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Constraints::FreeModifications(tModifications* modifications)
{
	tModifications::iterator iterMod = modifications->begin();
	while (iterMod != modifications->end())
	{
		Modification* modification = *iterMod;
		switch (modification->fTagModification)
		{

		case eDeleted:
		case eInserted:
		case eModified:
		{
			delete modification;
			break;
		}

		default:
			assert (false);
		}
	}

	delete modifications;
}
} // namespace

