// Constraints.h: interface for the Constraints class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONSTRAINTS_H)
#define AFX_CONSTRAINTS_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>
#include <vector>

#include "oti.h"
#include "otivector.h"

#include "Variables.h"
#include "UCProblemInstance.h"

namespace ucf
{

class Constraints  
{
public:
	Constraints();
	virtual ~Constraints();

	typedef int tIndex;
	typedef int tTime;
	typedef double tNumber;
	typedef std::vector<tNumber> tDenseVector;
	typedef oti::SparseVector<tNumber> tSparseVector;
	typedef int tGlobalName;
	typedef std::vector<tGlobalName> tGlobalNames;
	
	typedef oti::OtiStatic::eConstrType eConstrType;

	void Initialize(
		const Variables* variables,
		const UCProblemInstance* problemInstance);

	void SetNumRows(tIndex numRows);

	tIndex GetNumRows() const { return fNumRows; }

	virtual void GetConstraint(
			tIndex whichConstr,
			tSparseVector& coefficients,
			eConstrType& constrType,
			tNumber& rhs, tNumber& range) const =0;

	/*
	Return the number of non zero in the constraints.
	*/
	tIndex GetNonZero() const;

	/*
	Set the solver index for the whichConstr-th constratin.
	*/
	virtual void SetConstraintGlobalIndex(
		tIndex wichConstr,
		int globalIndex);

	/*
	Extract from a dense vector of duals, relative to the whole problem,
	the sparse vector of constraints dual.
	*/
	virtual void ExtractDuals(
		const tDenseVector& allDuals,
		tSparseVector& constrDuals) const;

	enum eModification
	{
		eDeleted = 0,
		eInserted,
		eModified
	};
	struct Modification
	{
		eModification fTagModification;
		tIndex fIndex;
	};
	typedef ActorModification<Modification> tActorModification;
	typedef tActorModification::tModifications tModifications;

	tIndex RegisterActor();
	void GetModification(
		tIndex actor, tModifications*& modifications) const;
	static void FreeModifications(tModifications* modifications);

	const Variables* GetVariables() const { return fVariables; }
	const UCProblemInstance* GetProblemInstance() const { return fProblemInstance; }

private:
	tIndex fNumRows;

	typedef std::vector<tActorModification*> tVarActors;

	const Variables* fVariables;
	const UCProblemInstance* fProblemInstance;

	/*
	fGlobalNames[i] is the row of the i-th constraint of this in the
	name space of the solver.
	*/
	tGlobalNames fGlobalNames;

	tIndex fFirstFreeActor;
	tVarActors fActors;
};

} // namespace

#endif // !defined(AFX_CONSTRAINTS_H)
