// ObjectiveFunction.h: interface for the ObjectiveFunction class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJECTIVEFUNCTION_H)
#define AFX_OBJECTIVEFUNCTION_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "otivector.h"
#include "Variables.h"

namespace ucf
{

class ObjectiveFunction
{
public:
	typedef int tIndex;
	typedef double tNumber;
	typedef oti::SparseVector<tNumber> tVector;

	enum eFunctionType
	{
		eLinear = 0,
		eQuadratic,
		eBlackBox
	};

	ObjectiveFunction();
	virtual ~ObjectiveFunction();

	void Initialize(const Variables* variables);

	virtual eFunctionType GetFunctionType() const =0;

	/*
	Return true if the has to minimize, else the objective function has
	to maximize.
	*/
	virtual bool GetMinimization() const =0;

	virtual void GetLinearCoeffs(tVector& coeffs) const =0;
	virtual void GetQuadCoeffs(tVector& coeffs, tIndex index) const =0;
	virtual tNumber Evaluate(const tVector& point) const =0;

	const Variables* GetVariables() const { return fVariables; }

private:
	const Variables* fVariables;
};

//////////////////////////////////////////////////////////////////////
// inlines
//////////////////////////////////////////////////////////////////////

inline void ObjectiveFunction::Initialize(const Variables* variables)
{
	fVariables = variables;
}

} // namespace

#endif // !defined(AFX_OBJECTIVEFUNCTION_H)
