// Variables.cpp: implementation of the Variables class.
//
//////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
// Disable the warning n. 4786
#pragma warning( disable : 4786 )
#endif

#include "Variables.h"

namespace ucf
{

//////////////////////////////////////////////////////////////////////
// Class Variables
//////////////////////////////////////////////////////////////////////

const Variables::tVarValue Variables::kInfinite(oti::Limits<tVarValue>::Infinity());

Variables::Variables()
	:fFirstFreeGroup(0)
	,fVarsGroups(8, static_cast<Group*>(NULL)) // preallocation
	,fGlobalNames(8, 0) // preallocation
	,fFirstFreeActor(0)
	,fVarActors(8, static_cast<tActorModification*>(NULL)) // preallocation
	,fNumVars(0)
{

}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

Variables::~Variables()
{

}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::RegisterVars(eVarType type, tIndex numOfVar, tIndex& groupName)
{
	if (fFirstFreeGroup>=tIndex(fVarsGroups.size()))
	{
		fVarsGroups.resize(fVarsGroups.size()*2, static_cast<Group*>(NULL)); //reallocation
		fGlobalNames.resize(fGlobalNames.size()*2, 0); //reallocation
		assert (fFirstFreeGroup<tIndex(fVarsGroups.size()));
	}
	assert (fVarsGroups.size() == fGlobalNames.size());

	Group* group = new Group(type, numOfVar);
	fVarsGroups[fFirstFreeGroup]=group;
	fGlobalNames[fFirstFreeGroup] = fNumVars;
	groupName = fFirstFreeGroup;

	fNumVars += numOfVar;
	++fFirstFreeGroup;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::SetVarRange(
	tIndex groupName, tIndex varName,
	tVarValue lowerBound, tVarValue upperBound)
{
	Group* group = fVarsGroups[groupName];
	group->ModifyLowerBound(varName, lowerBound);
	group->ModifyUpperBound(varName, upperBound);
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::EndRegistration()
{
	// Nothing to sdo.
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::ToLocalIndex(
		tIndex globalName, tIndex& groupName, tIndex& varName) const
{
	if (fFirstFreeGroup==0)
	{
		// Nothing to do
		groupName = 0;
		varName = 0;
		return;
	}

	tIndex i=0;
	tIndex j=fFirstFreeGroup-1;
	while (i!=j)
	{
		tIndex k = (i+j)/2;
		if (fGlobalNames[k]==globalName)
			i = j = k+1;
		else if (fGlobalNames[k] > globalName)
			j = k;
		else // if (fGlobalNames[k] < globalName)
			i = k;
	}

	assert (i>=0  && i<fFirstFreeGroup);
	groupName = i;
	varName = globalName - fGlobalNames[i] - 1;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::SetVar(tIndex groupName, tIndex varName, tVarValue value)
{
	fVarsGroups[groupName]->SetVar(varName, value);

	for (tIndex i = 0; i<fFirstFreeActor; i++)
	{
		Modification* modification = new FixSetVarMod
			(eSetVar, groupName, varName, value);

		fVarActors[i]->RegisterModification(modification);
	}
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::FixVar(tIndex groupName, tIndex varName, tVarValue value)
{
	fVarsGroups[groupName]->FixVar(varName, value);

	for (tIndex i = 0; i<fFirstFreeActor; i++)
	{
		Modification* modification = new FixSetVarMod
			(eFixedVar, groupName, varName, value);

		fVarActors[i]->RegisterModification(modification);
	}
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::FreeVar(tIndex groupName, tIndex varName)
{
	fVarsGroups[groupName]->FreeVar(varName);

	for (tIndex i = 0; i<fFirstFreeActor; i++)
	{
		Modification* modification = new UnfixVarMod
			(eFreedVar, groupName, varName);

		fVarActors[i]->RegisterModification(modification);
	}
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

Variables::tIndex Variables::RegisterActor()
{
	if (fFirstFreeActor>=tIndex(fVarActors.size()))
	{
		fVarActors.resize(fVarActors.size()*2, NULL);
	};

	fVarActors[fFirstFreeActor] = new tActorModification;

	return fFirstFreeActor++;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

void Variables::FreeModifications(tModifications* modifications)
{
	tModifications::iterator iterMod = modifications->begin();
	while (iterMod != modifications->end())
	{
		Modification* modification = *iterMod;
		switch (modification->fTagModification)
		{

		case eSetVar:
		case eFixedVar:
			{
				FixSetVarMod* fixSetVarMod = static_cast<FixSetVarMod*>(modification);
				delete fixSetVarMod;
				break;
			}

		case eFreedVar:
			{
				UnfixVarMod* unfixVarMod = static_cast<UnfixVarMod*>(modification);
				delete unfixVarMod;
				break;
			}

		default:
			assert (false);
		}
	}

	delete modifications;
}


} //namespace
